// Copyright (c) 97 Andrew Maltsev <am@amsoft.ru>
// All rights reserved
// 
// mSQL v2.* driver for EasySQL
//
// mSQL v2.* is a lightweight SQL engine written by D.Hughes.
// It is available from:
//  http://www.Hughes.com.au/
//
// Attention: This driver is to be used with 2.x versions of mSQL, it
// will not work with mSQL 1.x.  Use `msql' driver to get access to mSQL
// 1.x databases instead of `msql2'.
//
#include <unistd.h>	// for write()
#include <msql.h>
#include "DrvHeader.h"

///////////////////////////////////////////////////////////////////////////////
// `This may change in the future' and it is not available from msql.h
// But it is only way to alter port/socket, so..
//
extern "C" { int msqlLoadConfigFile(char const *); }

///////////////////////////////////////////////////////////////////////////////
// Data structure
//
struct DS(data) : public easysql_error_reporting
{ int sock;
//
  DS(data)();
  ~DS(data)();
};

///////////////////////////////////////////////////////////////////////////////
// Creating mSQL driver instance
//
DS(data)::DS(data) ()
         :easysql_error_reporting(DriverName "_driver",D_IDRV)
         ,sock(-1)
{
}

///////////////////////////////////////////////////////////////////////////////
// Destroing mSQL driver instance
//
DS(data)::~DS(data) ()
{
}

///////////////////////////////////////////////////////////////////////////////
// Connecting to mSQL server
//
bool
DS(connect) (DS(data) *data,
             char const *(*values)(void const *, char const *),
             void const *that)
{
// Checking for alternative configuration file
//
  if(values(that,"config"))
   msqlLoadConfigFile(values(that,"config"));

// Checking db name presence
//
  char const *db=values(that,"database");
  if(!db)
   { data->set_error("connect(): database name required");
     return false;
   }

// Connecting to mSQL engine
//
  char const *host=values(that,"host");
  data->sock=msqlConnect((char *)host);
  if(data->sock==-1)
   { data->set_error("connect(host=\"%s\"): %s",host,msqlErrMsg);
     return false;
   }

// Selecting database
//
  if(msqlSelectDB(data->sock,(char *)db)==-1)
   { data->set_error("connect(db=\"%s\"): %s",db,msqlErrMsg);
     return false;
   }

  log(D_IDRV|D_L2,"Connected to mSQL engine (%s,%s), socket=%d",host,db,data->sock);
  return true;
}

///////////////////////////////////////////////////////////////////////////////
// Checking connection
//
bool
DS(check_connection) (DS(data) *data)
{ if(data->sock==-1) return false;
  if(write(data->sock,data,0)!=0)
   { data->set_error("msql2 server has gone, closing (socket=%d)",data->sock);
     DS(drop)(data);
     return false;
   }
  return true;
}

///////////////////////////////////////////////////////////////////////////////
// Dropping connection to engine
//
bool
DS(drop) (DS(data) *data)
{ if(data->sock!=-1)
   { msqlClose(data->sock);
     data->sock=-1;
   }
  return true;
}

///////////////////////////////////////////////////////////////////////////////
// Sending query
//
bool
DS(query) (DS(data) *data, esqlResult &r, char const *q)
{ log(D_IDRV|D_L1,"query(\"%s\")",q);
  r=NULL;

  if(msqlQuery(data->sock,(char *)q)==-1)
   { data->set_error("query(\"%s\"): %s",q,msqlErrMsg);
     return false;
   }

  r=(esqlResult)msqlStoreResult();
  return true;
}

///////////////////////////////////////////////////////////////////////////////
// Freeing query results
//
bool
DS(rfree) (DS(data) *, esqlResult r)
{ msqlFreeResult((m_result *)r);
  return true;
}

///////////////////////////////////////////////////////////////////////////////
// Fetching next record from query results
//
bool
DS(row) (DS(data) *, esqlRow &row, esqlResult r)
{ row=(esqlRow)msqlFetchRow((m_result *)r);
  return true;
}

///////////////////////////////////////////////////////////////////////////////
// Driver's and engine version
//
char const *
DS(version) (DS(data) *data)
{ static char *ver=NULL;
  if(!ver) ver=(typeof(ver))mAlloc(100);
  snprintf(ver,100,"%s %s",DriverName,EASYSQL_VERSION);
  if(DS(check_connection)(data))
   { char const *v=msqlGetServerInfo();
     unsigned p=msqlGetProtoInfo();
     if(v && strlen(v)+strlen(ver)+10<100)
      snprintf(ver+strlen(ver),100-strlen(ver)-1," %u %s",p,v);
   }
  return ver;
}

///////////////////////////////////////////////////////////////////////////////
// Connection and engine status. Due to inability to get any
// human-readable status from the engine we will two predefined messages
//
char const *
DS(status) (DS(data) *data)
{ return DS(check_connection)(data) ? msqlGetHostInfo() : "Not connected to engine";
}

///////////////////////////////////////////////////////////////////////////////
// Work around for stupid declaration in msql.h
// Should be removed if declaration will be fixed later.
//
void const *DS(foo)=msqlTypeNames;

///////////////////////////////////////////////////////////////////////////////
// Standart functions
//
#include "../DrvStd.h"

///////////////////////////////////////////////////////////////////////////////
R("$Id: msql2.C,v 1.1 1997/11/05 19:51:52 am Exp $");
