// Copyright (c) 1997 Andrew Maltsev <am@amsoft.ru>
// All rights reserved
// 
// Queries register. A nice helper, very usable to track down lost rfree
// calls.
//
#include "es_int.h"

///////////////////////////////////////////////////////////////////////////////
// A portion, by which queries slots allocated. This is also a water
// mark, by which a warning issued about `too many active queries'.
//
#define Chunk 10

///////////////////////////////////////////////////////////////////////////////
// Creating an empty register
//
easysql_queries_register::easysql_queries_register(easysql_driver_shell *drv)
                         :list(NULL)
                         ,size(0)
                         ,driver(drv)
{
}

///////////////////////////////////////////////////////////////////////////////
// Destroying. If there are active queries in register they are rfree'd
// and reported as errors to log
//
easysql_queries_register::~easysql_queries_register()
{ for(unsigned i=0; i!=size; i++)
   if(list[i].active)
    { log(D_ERROR|D_MISC,"Unfreed query %p, freeing..",list[i].r);
      driver->rfree(list[i].r);
    }
  Free(list);
}

///////////////////////////////////////////////////////////////////////////////
// Registering new query. Just looks for non-active slot or allocates
// new chunk of slots.
//
void
easysql_queries_register::add(esqlResult r)
{ log(D_MISC|D_L4,"Registering SQL query %p",r);
  for(unsigned i=0; i!=size; i++)
   if(!list[i].active)
    { list[i].active=true;
      list[i].r=r;
      return;
    }
  if((size%Chunk) == 0)
   { if(size)
      log(D_ERROR|D_MISC,"It's already %u active queries. You forgot to call rfree?",size);
     list=(typeof(list))reAlloc(list,sizeof(*list)*(size+Chunk));
   }
  list[size].active=true;
  list[size++].r=r;
}

///////////////////////////////////////////////////////////////////////////////
// Dropping query info from register. It is just marks a slot as !active
// without real reallocations
//
bool
easysql_queries_register::drop(esqlResult r)
{ log(D_MISC|D_L4,"Freeing SQL query %p",r);
  for(unsigned i=0; i!=size; i++)
   if(list[i].active && list[i].r==r)
    { list[i].active=false;
      return true;
    }
  log(D_ERROR|D_MISC,"Attempt to rfree() not previously returned results (%p)",r);
  return false;
}

///////////////////////////////////////////////////////////////////////////////
R("$Id: query_reg.C,v 1.2 1998/05/03 17:22:26 am Exp $");
